import os 
class NoteReader:
    def __init__(self,config=None):
        self._config = config.load_config()

    def get_config(self):
        return self._config

    def read_note(self,path:str=None) -> str:
        lines:list = None
        with open(path, mode='r') as f:
            lines = f.readlines()
        return lines
        



from abc import ABC,abstractmethod
class SUPPORTEDFORMATSSTORAGE(ABC):
    @abstractmethod
    def get_all(self,storage=None) -> list:
        pass

    @abstractmethod 
    def get_by_name(self,name:str=None) -> dict:
        pass




from autoquiz.noteintroducer.protocols.note_reader_config import NOTEREADERCONFIG

class FormatExpected:
    def __init__(self,name:str=None,
                 title:str=None,
                 topic:str=None):
        self._name = name
        self._title = title
        self._topic = topic 

    def get_name(self) -> str:
        return self._name

    def get_title(self) -> str:
        return self._title

    def get_topic(self) -> str:
        return self._topic

class NoteReaderConfig(NOTEREADERCONFIG):
    def __init__(self):
        self._format = None

    def load_config(self):
        return self._format

    def set_expected_note_format(self,format:FormatExpected=None):
        self._format = format


class FormatExpectedBuilder:
    def __init__(self):
        self._name:str = None
        self._title:str = None
        self._topic:str = None

    def set_name(self,name):
        self._name = name 
        return self

    def set_title(self,title):
        self._title = title
        return self

    def set_topic(self,topic):
        self._topic = topic
        return self

    def build(self):
        return FormatExpected(name=self._name,
                              title=self._title,
                              topic=self._topic)



class Note:
    def __init__(self,content:str=None,tags:list=None):
        if content is None or content.rstrip()=="":
            raise VoidContentNoteException()
        self._content = content
        self._tags = tags

    def get_tags(self):
        return self._tags

    def get_content(self):
        return self._content


class VoidContentNoteException(BaseException):
    pass



from autoquiz.noteintroducer.format_selector import FormatSelector

def select_format(format:str=None,
                  format_storage=None,
                  note_reader_config=None,
                  format_selector = None):
    if None in [format,format_storage,note_reader_config]:
        raise ValueError()
    form_sel = FormatSelector(format_storage=format_storage,
                              note_reader_config=note_reader_config)
    form_sel.select_expected_note_format(format)


def display_list_of_supported_formats(format_storage_memory=None):
    serv = FormatSelector(format_storage=format_storage_memory)
    return serv.get_all_format()




class RecieveNoteDTO:
    def __init__(self,content:str=None,tags:list=None):
        if (content,tags) is None:
            raise ValueError()
        self._content = content 
        self._tags = tags

    def get_content(self):
        return self._content

    def get_tags(self):
        return self._tags



from abc import ABC,abstractmethod
class NOTEREADERCONFIG(ABC):

    @abstractmethod
    def load_config(self):
        self._value =True
        return self._format

    @abstractmethod
    def set_expected_note_format(self,format=None):
        self._format = format





from autoquiz.noteintroducer.agents.note_reader_config import FormatExpectedBuilder

class FormatSelector:
    def __init__(self,format_storage=None,
                 note_reader_config=None):
        self._note_reader_config = note_reader_config
        self._load_supported_formats(format_storage=format_storage)

    def get_all_format(self):
        return self._supported_format

    def _load_supported_formats(self,format_storage=None):
        self._supported_format = format_storage.get_all()

    def select_expected_note_format(self,expected:str=None):
        chos_format = FormatExpectedBuilder()
        for form_name,format in self._supported_format.items():
            if form_name==expected:
                chos_format.set_name(form_name)
                chos_format.set_title(format['title'])
                chos_format.set_topic(format['topic'])


        self._note_reader_config.set_expected_note_format(format=chos_format.build())




from autoquiz.noteintroducer.protocols.supported_formats_storage import SUPPORTEDFORMATSSTORAGE
import tomllib

class MemorySupportedFormat(SUPPORTEDFORMATSSTORAGE):

    def __init__(self,storage=None):
        if storage == None:
            self._storage = self._get_default(path="autoquiz/noteintroducer/data/default_formats.toml")
        else:
            self._storage = storage

    def get_all(self,storage=None):
        return self._storage

    def get_by_name(self,name=None):
        for storage in self._storage:
            if storage["name"]==name:
                return storage
    
    def _get_default(self,path:str=None):
        with open(path,'rb') as f:
            return tomllib.load(f)
